using System;
using System.Collections.Generic;
using System.Text;
using ceometric.ComputationalGeometry;
using ceometric.VectorGeometry;

//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
// You must reference both V# and G# first to run this program
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

namespace SurfaceReconstruction
{
    /// <summary>
    /// Reconstrucuts a triangulated surface from unordered edges
    /// defining triangular and and quad regions.
    /// </summary>
    class Program
    {
        static void Main(string[] args)
        {
            //---------------------------------------------------------------------
            //Settings for best results.
            ceometric.VectorGeometry.Global.AbsoluteEpsilon = -1;
            ceometric.VectorGeometry.Global.RelativeEpsilon = -1;

            //---------------------------------------------------------------------
            //Generate input edge data. This is done by reading triangles from an
            //.off file and extracting the edges of the triangles into a list.
            //The edge list will be randomly permuted and contains many duplicate
            //edges, which is OK for the algorithm.
            //
            //SPECIFY THE CORRECT FILE FORMAT HERE:
            //shark_en.xyz has comma (",") decimal seperator
            //shark_us.xyz has point (".") decimal seperator
            OFFReader or = new OFFReader();
            or.Read(@"..\..\shark_en.off");
            List<Edge> wireframeEdges = GetEdges(or.Triangles); //Method to get the edges of the triangles

            //---------------------------------------------------------------------
            //Reconstruct surface. The method finds all triangular and quad
            //regions in the wireframe and outputs a triangulated surface.
            KernelTimeGauge tg = new KernelTimeGauge();
            tg.Start();
            SurfaceReconstructor sr = new SurfaceReconstructor(wireframeEdges);
            sr.ReconstructSurface(6); //6 decimal places precision
            int failed = 0;
            List<Triangle> reconstructedTriangles = sr.GetSurfaceAsTriangleList(ref failed);
            tg.Stop();

            //---------------------------------------------------------------------
            //Some statistics
            Console.WriteLine("Triangles read from off: " + or.Triangles.Count);
            Console.WriteLine("Edges extracted        : " + wireframeEdges.Count);
            Console.WriteLine("Triangles reconstructed: " + reconstructedTriangles.Count +
                " in " + tg.DurationInMilliSeconds.ToString() + "ms");

            //---------------------------------------------------------------------
            // Visualize in dxf format
            // Get the free DWG TrueView viewer at www.autodesk.com to view dxf
            Scene sc = new Scene();
            Scene.Layer lay = new Scene.Layer("reconstructed_surface", 5);

            // Add objects to the scene
            sc.Add(reconstructedTriangles, lay);

            // Output the scene as dxf
            string outfile = @"..\..\reconstructed_surface.dxf";
            sc.WriteDxf(outfile);
            Console.WriteLine("dxf written to " + outfile);

            Console.WriteLine("Hit any key to continue.");
            Console.ReadLine();
        }

        /// <summary>Extracts the edges from a list of triangles without removing duplicates.</summary>
        /// <param name="trs">A list of triangles.</param>
        /// <returns>Returns a list of randomly permuted edges.</returns>
        public static List<Edge> GetEdges(List<Triangle> trs)
        {
            List<Edge> edges = new List<Edge>();
            for (int i = 0; i < trs.Count; i++)
            {
                edges.Add(trs[i].Edge1);
                edges.Add(trs[i].Edge2);
                edges.Add(trs[i].Edge3);
            }
            Edge.RandomPermute(ref edges);

            return edges;
        }
    }
}